from __future__ import annotations

# Chinese CLIP models
MODELS = {
    "CN-CLIP/ViT-B-16",
    "CN-CLIP/ViT-L-14",
    "CN-CLIP/ViT-L-14-336",
    "CN-CLIP/ViT-H-14",
    "CN-CLIP/RN50",
}

raise NotImplementedError("CNCLIP is not implemented yet.")
