from __future__ import annotations

# Multi-lingual CLIP models
MODELS = {
    "M-CLIP/XLM-Roberta-Large-Vit-B-16Plus",
    "M-CLIP/M-BERT-Distil-40",
    "M-CLIP/M-BERT-Base-ViT-B",
    "M-CLIP/XLM-Roberta-Large-Vit-B-32",
    "M-CLIP/XLM-Roberta-Large-Vit-L-14",
}

raise NotImplementedError("MCCLIP is not implemented yet.")
